@@ -16,8 +16,10 @@ char *cgi_value P_((cgi_form *formp, char *name));
 int cgi_output_value P_((cgi_form *formp, char *name, char *formatstr, FILE *outstream));
 void cgi_concat_errno P_((char *string));
 int cgi_template_fill P_((cgi_form *formp, char *templatefile));
+void cgi_pclose_fix P_((void));
 int cgi_mail_template P_((cgi_form *formp, char *templatefile));
 void cgi_output_failure P_((cgi_form *formp, char *msg));
+char *cgi_char2entity P_((int c));
 void cgi_ascii2html P_((FILE *instream, FILE *outstream));
 void cgi_redirect P_((char *url));
 void cgi_output_success P_((cgi_form *formp, char *msg));
@@ -29,7 +31,7 @@ char *cgi_next_line_in_string P_((char *string, char *linebuf, int linebuflen));
 int cgi_cso_field P_((char *linebuf, char *fieldname, char *valuebuf, int buflen));
 void cgi_cso_header P_((cgi_form *formp, char *query));
 void cgi_cso_footer P_((void));
-void cgi_url_hack P_((char linebuf[]));
+void cgi_url_print P_((char linebuf[]));
 int cgi_standard_cso P_((void));
 
 /* finger.c */
