@@ -562,7 +562,7 @@ cgi_mail_template(formp, templatefile)
   int retval;
   char buf[BUFSIZ], command[BUFSIZ];
   int nbytes;
-  char *fromhost, *servername, datebuf[80], errorfile[L_tmpnam];
+  char *envvar, datebuf[80], errorfile[L_tmpnam];
   int old_stdout, old_stderr, errorfd;
   time_t rightnow;
   struct tm *timeptr;
@@ -611,28 +611,46 @@ cgi_mail_template(formp, templatefile)
   /* put "Received: from HOST with HTTP;\n\tdate\n" */
   /* See RFC 822 appendix D. */
 
-  fromhost = getenv("REMOTE_HOST");
-  if (!fromhost || !(*fromhost)) fromhost = getenv("REMOTE_ADDR");
-  if (fromhost && *fromhost)
+  envvar = getenv("REMOTE_HOST");
+  if (!envvar || !(*envvar)) envvar = getenv("REMOTE_ADDR");
+  if (envvar && *envvar)
     {
       fputs("Received: from ", mypipe);
-      fputs(fromhost, mypipe);
-      servername=getenv("SERVER_NAME");
-      if (servername && *servername)
+      fputs(envvar, mypipe);
+      envvar=getenv("SERVER_NAME");
+      if (envvar && *envvar)
 	{
 	  fputs(" by ", mypipe);
-	  fputs(servername, mypipe);
+	  fputs(envvar, mypipe);
 	}
       fputs(" with HTTP;\n", mypipe);
 
       /* Put date-time (RFC 822 sec. 5, RFC 1123 sec. 5.2.14) */
-      /* XXX not compliant if lang is not English or TZ unknown */
+      /* FIXME: not compliant if lang is not English or TZ unknown */
       time(&rightnow);
       timeptr=localtime(&rightnow);
       strftime(datebuf, 79, "\t%a, %d %b %Y %H:%M:%S %Z\n", timeptr);
       fputs(datebuf, mypipe);
     }
 
+#ifdef ENABLE_XHEADERS
+  /* Add the X-Mailer: header */
+  fputs("X-Mailer: cgiemail ", mypipe);
+  fputs(CGIEMAIL_RELEASE, mypipe);
+  if (NULL != (envvar = getenv("HTTP_REFERER")))
+    {
+      fputs("\n\t(form=\"", mypipe);
+      fputs(getenv("HTTP_REFERER"), mypipe);
+      fputs("\")", mypipe);
+    }
+  /* SCRIPT_NAME is always set; skip envvar check */
+  fputs("\n\t(action=\"", mypipe);
+  fputs(getenv("SCRIPT_NAME"), mypipe);
+  if (NULL != (envvar = getenv("PATH_INFO")))
+    fputs(getenv("PATH_INFO"), mypipe);
+  fputs("\")\n", mypipe);
+#endif
+
   /* send filled-in template */
   rewind(formp->tmpf);
   do {
@@ -643,29 +661,32 @@ cgi_mail_template(formp, templatefile)
   /* close */
   retval=pclose(mypipe);
 
-  /*
-   * The return value of pclose is not always meaningful.
-   * However, sendmail does not write to stdout/stderr if successful.
-   */
+  if (retval)
+    {
+      /*
+       * The return value of pclose is not always meaningful.
+       * However, sendmail usually does not write to stdout/stderr on success.
+       */
 
-  /* If we can't open the errorfile, something went wrong. */
-  errorfp = fopen(errorfile, "r");
-  if (!errorfp)
-    {
-      formp->errcond=1;
-      sprintf(formp->errmsg,
-	     "500 sendmail exit %d - check httpd error logs", retval);
-      strcpy(formp->errinfo, command);
-    }
-  else
-    {
-      /* See if there was any error message */
-      fread(formp->errinfo, sizeof(char), CGI_ERRMSG_MAX, errorfp);
-      if (cgi_nonblank(formp->errinfo))
+      /* If we can't open the errorfile, something went wrong. */
+      errorfp = fopen(errorfile, "r");
+      if (!errorfp)
 	{
 	  formp->errcond=1;
-	  sprintf(formp->errmsg, "500 sendmail exit %d with error message",
-		 retval);
+	  sprintf(formp->errmsg,
+		  "500 sendmail exit %d - check httpd error logs", retval);
+	  strcpy(formp->errinfo, command);
+	}
+      else
+	{
+	  /* See if there was any error message */
+	  fread(formp->errinfo, sizeof(char), CGI_ERRMSG_MAX, errorfp);
+	  if (cgi_nonblank(formp->errinfo))
+	    {
+	      formp->errcond=1;
+	      sprintf(formp->errmsg, "500 sendmail exit %d with error message",
+		      retval);
+	    }
 	}
     }
 
