@@ -119,20 +119,26 @@ cgi_cso_footer()
 }
 
 void
-cgi_url_hack(linebuf)
+cgi_url_print(linebuf)
      char linebuf[];
 {
-  if (!strncmp(linebuf, "       url:", 11))
+  /* Look for URL field or any field like http://... */
+  if (!strncmp(linebuf, "       url:", 11) || strstr(linebuf, "://"))
     {
-      char urlbuf[256], *ptr1;
+      char *ptr1;
 
-      strcpy(urlbuf, linebuf+12);
-      /* remove trailing whitespace */
-      for(ptr1=urlbuf+strlen(urlbuf)-1;
-	  isspace(*ptr1) && ptr1 > urlbuf;
-	  ptr1--) *ptr1 = '\0';
+      fwrite(linebuf, 1, 12, stdout);
 
-      printf("%11.11s <a href=\"%s\">%s</a>\n", linebuf, urlbuf, urlbuf);
+      /* make the URL into a hyperlink */
+      fputs("<a href=\"", stdout);
+      for(ptr1=linebuf+12; *ptr1 && !isspace(*ptr1); ptr1++)
+	putchar((int) *ptr1);
+      fputs("\">", stdout);
+      fwrite(linebuf+12, 1, ptr1-(linebuf+12), stdout);
+      fputs("</a>", stdout);
+
+      /* print the rest of the line */
+      puts(ptr1);
     }
   else puts(linebuf);
   return;
@@ -197,8 +203,13 @@ cgi_standard_cso()
   if (!ptr)
     {
       cgi_cso_header(&form, query);
-      puts("<PRE>");
-      ptr=fingerbuf;
+      linebuf[0]='\0';
+      ptr=strstr(fingerbuf, "There was 1 match to your request");
+      if (!ptr) ptr = fingerbuf;
+      motdlen = ptr - fingerbuf;
+      if (motdlen) ptr=cgi_next_line_in_string(ptr, linebuf, 256);
+      printf("<p>%s</p>\n<PRE>", linebuf);
+
       while ((ptr=cgi_next_line_in_string(ptr, linebuf, 256)) != (char *)0)
 	{
 	  /* Skip alias field */
@@ -214,81 +225,65 @@ cgi_standard_cso()
 	      continue;
 	    }
 
-	  /* make hyperlink in url: field */
-	  if (cgi_cso_field(linebuf, "url", urlbuf, 256))
-	    {
-	      printf("%10s: <A HREF=\"%s\">%s</A>\n", "url", urlbuf, urlbuf);
-	      continue;
-	    }
-
-	  /* Otherwise print the line verbatim */
-	  puts(linebuf);
+	  /* make hyperlink if appropriate; otherwise print the line */
+	  cgi_url_print(linebuf);
 	}
-      puts("</PRE>");
-      cgi_cso_footer();
-      return(0);
     }
-
-  /* found -- print nice HTML list of matches */
-  deptbuf[0] = '\0';		/* no dept. found yet */
-  cgi_cso_header(&form, query);
-  puts(linebuf);
-  puts("<P>\n<PRE>");
-  while ((ptr=cgi_next_line_in_string(ptr, linebuf, 256)) != (char *)0)
+  else
     {
-      if (cgi_cso_field(linebuf, "department", deptbuf, 256)) continue;
-      if (cgi_cso_field(linebuf, "name", namebuf, 256)) continue;
-      if (cgi_cso_field(linebuf, "title", titlebuf, 256)) continue;
-      if (cgi_cso_field(linebuf, "year", yearbuf, 256)) continue;
-      if (cgi_cso_field(linebuf, "alias", aliasbuf, 256))
+      /* found -- print nice HTML list of matches */
+      deptbuf[0] = '\0';		/* no dept. found yet */
+      cgi_cso_header(&form, query);
+      printf("<p>%s</p>\n<PRE>", linebuf);
+      while ((ptr=cgi_next_line_in_string(ptr, linebuf, 256)) != (char *)0)
 	{
-	  /* put hyperlink to more specific query */
-	  fputs("<A HREF=\"", stdout);
-	  fputs(getenv("SCRIPT_NAME"), stdout);
-	  fputs("?query=alias%3D", stdout);
-	  fputs(aliasbuf, stdout);
-	  fputs("\">", stdout);
-	  fputs(namebuf, stdout);
-
-	  /* put periods up to next field */
-	  fputs("</A>...", stdout);
-	  fieldlen = 24-strlen(namebuf);
-	  for(i=0; i<fieldlen; i++) putchar((int) '.');
-
-	  /* put "dept, year n", "dept", or "Year n" as appropriate */
-	  /* fall back on "title" if available */
-	  if (*deptbuf)
+	  if (cgi_cso_field(linebuf, "department", deptbuf, 256)) continue;
+	  if (cgi_cso_field(linebuf, "name", namebuf, 256)) continue;
+	  if (cgi_cso_field(linebuf, "title", titlebuf, 256)) continue;
+	  if (cgi_cso_field(linebuf, "year", yearbuf, 256)) continue;
+	  if (cgi_cso_field(linebuf, "alias", aliasbuf, 256))
 	    {
-	      if (*yearbuf)
+	      /* put hyperlink to more specific query */
+	      fputs("<A HREF=\"", stdout);
+	      fputs(getenv("SCRIPT_NAME"), stdout);
+	      fputs("?query=alias%3D", stdout);
+	      fputs(aliasbuf, stdout);
+	      fputs("\">", stdout);
+	      fputs(namebuf, stdout);
+
+	      /* put periods up to next field */
+	      fputs("</A>...", stdout);
+	      fieldlen = 24-strlen(namebuf);
+	      for(i=0; i<fieldlen; i++) putchar((int) '.');
+
+	      /* put "dept, year n", "dept", or "Year n" as appropriate */
+	      /* fall back on "title" if available */
+	      if (*deptbuf)
 		{
-		  fputs(deptbuf, stdout);
-		  fputs(", year ", stdout);
-		  puts(yearbuf);
+		  if (*yearbuf)
+		    {
+		      fputs(deptbuf, stdout);
+		      fputs(", year ", stdout);
+		      puts(yearbuf);
+		    }
+		  else puts(deptbuf);
 		}
-	      else puts(deptbuf);
+	      else
+		if (*yearbuf)
+		  {
+		    fputs("Year ", stdout);
+		    puts(yearbuf);
+		  }
+		else
+		  if (*titlebuf) puts(titlebuf);
+		  else puts("");
+	      deptbuf[0] = titlebuf[0] = yearbuf[0] = '\0';
 	    }
-	  else
-	    if (*yearbuf)
-	      {
-		fputs("Year ", stdout);
-		puts(yearbuf);
-	      }
-	    else
-	      if (*titlebuf) puts(titlebuf);
-	      else puts("");
-	  deptbuf[0] = titlebuf[0] = yearbuf[0] = '\0';
 	}
     }
 
   puts("</PRE>");
-
-  if (motdlen > 1)
-    {
-      /* print motd that precedes data */
-      puts("\n<HR>\n");
-      fwrite(fingerbuf, sizeof(char), motdlen, stdout);
-    }
-
+  fwrite(fingerbuf, sizeof(char), motdlen, stdout);
   cgi_cso_footer();
   return(0);
 }
