@@ -7,8 +7,9 @@
  **********************************************************************/
 #include "mit-copyright.h"
 
-static char rcsid[]="$Header: /mit/wwwdev/src/cgi/RCS/csoget.c,v 1.6 1996/10/03 13:36:34 brlewis Exp $";
+static char rcsid[]="$Header: /mit/wwwdev/src/cgi/RCS/csoget.c,v 1.9 1997/05/29 21:26:47 brlewis Exp $";
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <netdb.h>
 #include <sys/types.h>
@@ -39,10 +40,6 @@ csoget(query, host, origbuf, origbufsize)
   sname.sin_family = AF_INET;
   memcpy(&(sname.sin_addr), hp->h_addr, hp->h_length);
 
-  /* Map underscores to spaces in query. */
-  for (idx=query; *idx; idx++)
-    if (*idx == '_') *idx = ' ';
-
   /* Try port 106 first (MITdir limiter), then 105 (standard port) */
   for (portnum=108; portnum >= 105; portnum -= 3)
     {
@@ -52,7 +49,8 @@ csoget(query, host, origbuf, origbufsize)
       sock = socket(PF_INET, SOCK_STREAM, 0);
 
       /* connect */
-      if (connect(sock, &sname, sizeof(sname)) < 0) continue;
+      if (connect(sock, (struct sockaddr *)&sname, sizeof(sname)) < 0)
+	continue;
 
       /* send ident */
       remote_addr = getenv("REMOTE_ADDR");
@@ -114,10 +112,6 @@ finger(query, host, buf, bufsize)
   /* MITdir vagary: Results might already be cleaned up */
   if (!isdigit(buf[0]) && buf[0] != '-') return(strlen(buf));
 
-  /* Remove leading trash */
-  if (!strncmp(buf, "-200:1:motd:", 12))
-    strcpy(buf, buf+12);
-
   /* Clean each line */
   for(textline1=buf;
       textline1 > (char *) 1;
@@ -132,13 +126,14 @@ finger(query, host, buf, bufsize)
 	  if (!strncmp(colon1-3, "200", 3)) /* query results */
 	    {
 	      colon2=strpbrk(1+colon1, ":\012");
-	      if (!colon2 || *colon2 == '\012')	/* "Ok." or "Bye!" */
+	      if (!colon2 || *colon2 == '\012'	/* "Ok." or "Bye!" */
+		  || !strncmp(colon2, ":motd:", 6)) /* skip motd */
 		{
 		  textline2=strchr(1+colon1, '\012' /* LF */);
 		  if (textline2)
 		    {
-		      /* Remove everything but the newline */
-		      strcpy(textline1, textline2);
+		      /* Remove everything */
+		      strcpy(textline1, textline2+1);
 		      repeat_line=1; /* don't skip a line */
 		    }
 		  else
@@ -170,3 +165,24 @@ finger(query, host, buf, bufsize)
     }
   return(strlen(buf));
 }
+
+#ifdef TEST_CSOGET
+
+main(argc, argv)
+     int argc;
+     char *argv[];
+{
+  char buf[BUFSIZ];
+  static char *sep="--------------------------------------------------------";
+
+  puts(sep);
+  csoget(argv[1], argv[2], buf, BUFSIZ);
+  puts(buf);
+  puts(sep);
+  finger(argv[1], argv[2], buf, BUFSIZ);
+  puts(buf);
+  puts(sep);
+  exit(0);
+}
+
+#endif
